@@ -32,6 +32,7 @@ print.stars_proxy = function(x, ..., n = 1e5, nfiles = 10, simplify = TRUE) {
 	if (!is.null(attr(x, "call_list"))) {
 		cat("call_list:\n")
 		print(unlist(attr(x, "call_list")))
+		cat("This object has pending lazy operations: dimensions as printed may not reflect this.\n")
 	}
 }
 
@@ -53,7 +54,7 @@ plot.stars_proxy = function(x, y, ..., downsample = get_downsample(dim(x))) {
 	plot(st_as_stars(x, downsample = downsample, ...), ..., downsample = 0)
 }
 
-st_stars_proxy = function(x, dimensions, ..., NA_value, resolutions, RasterIO = list()) {
+st_stars_proxy = function(x, dimensions, ..., NA_value, resolutions, RasterIO = list(), file_dim = NULL) {
 	stopifnot(!missing(NA_value))
 	stopifnot(!missing(resolutions))
 	stopifnot(length(list(...)) == 0)
@@ -62,7 +63,7 @@ st_stars_proxy = function(x, dimensions, ..., NA_value, resolutions, RasterIO =
 	if (length(RasterIO) == 0)
 		RasterIO = NULL
 	structure(x, dimensions = dimensions, NA_value = NA_value, resolutions = resolutions,
-		RasterIO = RasterIO, class = c("stars_proxy", "stars"))
+		RasterIO = RasterIO, file_dim = file_dim, class = c("stars_proxy", "stars"))
 }
 
 add_resolution = function(lst) {
@@ -77,31 +78,41 @@ add_resolution = function(lst) {
 	structure(lst, resolutions = resolutions)
 }
 
-
 #' @export
 #' @param along_crs logical; if \code{TRUE}, combine arrays along a CRS dimension
 #' @name c.stars
 c.stars_proxy = function(..., along = NA_integer_, along_crs = FALSE, try_hard = FALSE, 
 						 nms = names(list(...)), tolerance = sqrt(.Machine$double.eps)) {
 	dots = list(...)
+	get_file_dim = function(dots) {
+			do.call(rbind, lapply(dots, attr, "file_dim"))
+	}
 	if (!all(sapply(dots, function(x) inherits(x, "stars_proxy"))))
 		stop("all arguments to c() should be stars_proxy objects")
+	rio = attr(dots[[1]], "RasterIO")
 
 	# Case 1: merge attributes of several objects by simply putting them together in a single stars object;
 	# dim does not change:
-	if (length(dots) == 1) # do nothing
+	if (length(dots) == 1 && length(along) == 1 && is.na(along)) # do nothing
 		dots[[1]]
 	else if (along_crs)
 		combine_along_crs_proxy(dots)
 	else if (identical(along, NA_integer_)) { 
 		if (identical_dimensions(dots))
-			st_stars_proxy(do.call(c, lapply(dots, unclass)), dimensions = st_dimensions(dots[[1]]),
-				NA_value = attr(dots[[1]], "NA_value"), resolutions = NULL)
+			st_stars_proxy(setNamesIfnn(do.call(c, lapply(dots, unclass)), nms),
+						   dimensions = st_dimensions(dots[[1]]), 
+						   NA_value = attr(dots[[1]], "NA_value"), 
+						   resolutions = NULL,
+						   file_dim = get_file_dim(dots),
+						   RasterIO = rio)
 		else if (identical_dimensions(dots, ignore_resolution = TRUE, tolerance = tolerance)) {
 			dots = add_resolution(dots)
-			st_stars_proxy(do.call(c, lapply(dots, unclass)), dimensions = st_dimensions(dots[[1]]),
-				resolutions = attr(dots, "resolutions"),
-				NA_value = attr(dots[[1]], "NA_value"))
+			st_stars_proxy(setNamesIfnn(do.call(c, lapply(dots, unclass)), nms),
+						   dimensions = st_dimensions(dots[[1]]), 
+						   resolutions = attr(dots, "resolutions"),
+						   NA_value = attr(dots[[1]], "NA_value"), 
+						   file_dim = get_file_dim(dots),
+						   RasterIO = rio)
 		} else {
 			# currently catches only the special case of ... being a broken up time series:
 			along = sort_out_along(dots)
@@ -117,13 +128,17 @@ c.stars_proxy = function(..., along = NA_integer_, along_crs = FALSE, try_hard =
 					"ignored subdataset(s) with dimensions different from first subdataset:", 
 					paste(which(!ident), collapse = ", "), 
 					"\nuse gdal_subdatasets() to find all subdataset names"))
-				setNames(st_stars_proxy(do.call(c, 
-						lapply(dots[ident], unclass)), dimensions = st_dimensions(dots[[1]]),
-						NA_value = attr(dots[[1]], "NA_value"), resolutions = NULL),
-						nms[ident])
+				if (!is.null(nms))
+					nms = nms[ident]
+				st_stars_proxy(setNamesIfnn(do.call(c, lapply(dots[ident], unclass)), nms),
+							   dimensions = st_dimensions(dots[[1]]), 
+							   NA_value = attr(dots[[1]], "NA_value"), 
+							   resolutions = NULL, 
+							   file_dim = get_file_dim(dots),
+							   RasterIO = rio)
 			}
 		}
-	} else {
+	} else { # arrange along "along" dimension:
 		if (is.list(along)) { # custom ordering of ... over dimension(s) with values specified
 			stop("for proxy objects, along argument as list is not implemented")
 		} else { # loop over attributes, abind them:
@@ -149,7 +164,9 @@ c.stars_proxy = function(..., along = NA_integer_, along_crs = FALSE, try_hard =
 			if (along_dim == length(d) + 1)
 				names(dims)[along_dim] = if (is.character(along)) along else "new_dim"
 			st_stars_proxy(ret, dimensions = dims, NA_value = attr(dots[[1]], "NA_value"),
-				resolutions = NULL)
+				resolutions = NULL,
+				file_dim = get_file_dim(dots),
+				RasterIO = rio)
 		}
 	}
 }
@@ -203,8 +220,13 @@ fetch = function(x, downsample = 0, ...) {
 	if (!is.null(bands)) {
 		if (!is.null(bands$values) && is.numeric(bands$values)) 
 			rasterio$bands = bands$values
-		else if (!is.na(bands$from) && !is.na(bands$to) && (bands$to - bands$from + 1) < length(x[[1]]))
+		else if (!is.na(bands$from) && !is.na(bands$to) 
+				 # && (bands$to - bands$from + 1) < length(x[[1]])
+			)
 			rasterio$bands = seq(bands$from, bands$to)
+		if (!is.null(rasterio$bands) && length(rasterio$bands) > 1 && 
+				length(rasterio$bands) == length(x[[1]])) # one band in each file
+			rasterio$bands = NULL # https://github.com/r-spatial/stars/issues/608
 	}
 
 	# do it:
@@ -224,7 +246,7 @@ fetch = function(x, downsample = 0, ...) {
 		} else
 			offset = c(0,0)
 		file_name = unclass(x)[[i]]
-		if (is.function(file_name)) # realise:
+		if (is.function(file_name)) # realise/evaluate:
 			file_name = file_name()
 		ret[[i]] = read_stars(file_name, RasterIO = rasterio, 
 			NA_value = attr(x, "NA_value") %||% NA_real_, normalize_path = FALSE,
@@ -245,7 +267,7 @@ fetch = function(x, downsample = 0, ...) {
 			list(new_dim = names(x))
 	
 	ret = if (length(ret) == 1)
-		st_redimension(ret[[1]], along = along)
+		st_redimension(ret[[1]], name = along)
 	else
 		do.call(c, lapply(ret, st_redimension, along = along))
 	
@@ -260,8 +282,15 @@ fetch = function(x, downsample = 0, ...) {
 		}
 
 	ret = unclass(ret)
-	for (i in seq_along(ret))
+	for (i in seq_along(ret)) {
+		file_dim = attr(x, "file_dim")
+		if (is.null(bands) && !is.null(file_dim) && ncol(file_dim) == length(dim(new_dim)) 
+				&& ncol(file_dim) == 3) { # https://github.com/r-spatial/stars/issues/596
+			r = new_dim[[3]]$from:new_dim[[3]]$to # FIXME: or else use $values?
+			ret[[i]] = ret[[i]][,,r]
+		}
 		dim(ret[[i]]) = dim(new_dim)
+	}
 	adrop(st_set_crs(st_stars(setNames(ret, names(x)), new_dim), st_crs(x)))
 }
 
@@ -303,7 +332,7 @@ st_as_stars.stars_proxy = function(.x, ..., downsample = 0, url = attr(.x, "url"
 		ret # nocov end
 	} else {
 		cl = attr(.x, "call_list")
-		# FIXME: this means we ALLWAYS process after (possibly partial) reading; 
+		# FIXME: this means we ALWAYS process after (possibly partial) reading; 
 		# there are cases where this is not right. Hence:
 		# TODO: only warn when there is a reason to warn.
 		if (!all(downsample == 0))
@@ -396,9 +425,15 @@ is.na.stars_proxy = function(x) {
 	collect(x, match.call(), "is.na", "x", env = environment())
 }
 
+#' @name stars_subset
 #' @export
 "[<-.stars_proxy" = function(x, i, downsample = 0, value) {
-	collect(x, match.call(), "[<-", c("x", "i", "value", "downsample"), env = environment())
+	# https://stackoverflow.com/questions/9965577/copy-move-one-environment-to-another
+	# copy the environment, to avoid side effect later on:
+	# FIXME: to investigate - should this be done to env in every call to collect()?
+	env = as.environment(as.list(environment(), all.names = TRUE)) # copies
+	parent.env(env) = parent.env(environment())
+	collect(x, match.call(), "[<-", c("x", "i", "value", "downsample"), env)
 }
 
 
@@ -439,27 +474,36 @@ merge.stars_proxy = function(x, y, ..., name = "attributes") {
 		else
 			NULL
 	}
+	rio = attr(x, "RasterIO")
+	dim_orig = dim(x)
 	mc = match.call()
 	lst = as.list(mc)
 	cl = attr(x, "call_list")
 	if (length(lst) < 3)
 		return(x) # 
-	if (missing(i)) # insert:
+	if (missing(i) | !"i" %in% names(lst)) # insert:
 		lst = c(lst[1:2], i = TRUE, lst[-(1:2)])
 	if (inherits(i, c("character", "logical", "numeric")) && is.null(cl)) {
 		if (!is.null(unclass(x)[[i]])) { # can/should be selected now:
 			if (!is.null(resolutions <- attr(x, "resolutions")))
 				resolutions = resolutions[i, ]
 			x = st_stars_proxy(unclass(x)[i], st_dimensions(x), NA_value = attr(x, "NA_value"),
-				resolutions = resolutions)
+				resolutions = resolutions, file_dim = attr(x, "file_dim"),
+				RasterIO = rio)
 			lst[["i"]] = TRUE # this one has been handled now
 		}
-		for (ix in 1:3) { # FIXME: process further dimensions than 3?
-			if (length(lst) >= 4 && !is.null(r <- get_range(lst[[4]]))) {
+		ix = 1
+		while (length(lst) >= 4) { # https://github.com/r-spatial/stars/issues/496
+			if (!is.null(r <- get_range(lst[[4]]))) {
 				attr(x, "dimensions")[[ix]]$from = r[1]
 				attr(x, "dimensions")[[ix]]$to = r[2]
-				lst[[4]] = NULL # remove
+				if(!is.null(attr(x, "dimensions")[[ix]]$values)) {
+					attr(x, "dimensions")[[ix]]$values <- 
+						attr(x, "dimensions")[[ix]]$values[r[1]:r[2]]
+				}
 			}
+			ix = ix + 1
+			lst[[4]] = NULL # eat/remove
 		}
 	} else if (crop && inherits(i, c("sf", "sfc", "stars", "bbox"))) {
 		x = st_crop(x, i, ..., collect = FALSE) # does bounding box cropping only
@@ -467,12 +511,37 @@ merge.stars_proxy = function(x, y, ..., name = "attributes") {
 			lst[["i"]] = TRUE # this one has been handled now
 	}
 
-	# return:
-	if (length(lst) == 3 && isTRUE(lst[["i"]]) && is.null(cl)) # all is done
-		x 
-	else # still processing the geometries inside the bbox:
-		collect(x, as.call(lst), "[", c("x", "i", "drop", "crop"), 
-			env = environment()) # postpone every arguments > 3 to after reading cells
+	# return or collect?
+	file_dim = attr(x, "file_dim") %||% matrix(dim(x)[1:2], 1)
+	n_file_dim = ncol(file_dim)
+	if (length(lst) == 3 && isTRUE(lst[["i"]]) && is.null(cl) &&  # drop a number of files in the lists of files?
+			length(x) && length(dim(x)) > n_file_dim && 
+			length(x[[1]]) == prod(dim_orig[-(seq_len(ncol(file_dim)))])) { # https://github.com/r-spatial/stars/issues/561
+		# select from the vectors of proxy object names:
+		get_ix = function(d) {
+			stopifnot(inherits(d, "dimension"))
+			if (!is.na(d$from))
+				seq(d$from, d$to)
+			else
+				d$values
+		}
+		d = st_dimensions(x)[-seq_len(n_file_dim)] # dimensions not in the file(s)
+		e = do.call(expand.grid, lapply(dim_orig[-seq_len(n_file_dim)], seq_len)) # all combinations
+		e$rn = seq_len(nrow(e)) # their index
+		f = do.call(expand.grid, lapply(d, get_ix)) # the ones we want
+		if (!requireNamespace("dplyr", quietly = TRUE))
+			stop("package dplyr required, please install it first") # nocov
+		sel = dplyr::inner_join(e, f, by = colnames(f))$rn
+		for (i in seq_along(x)) # select:
+			x[[i]] = x[[i]][sel]
+		x
+	} else { # still processing the geometries inside the bbox:
+		if (length(lst) == 3 && isTRUE(lst[["i"]]) && is.null(cl))
+			x
+		else
+			collect(x, as.call(lst), "[", c("x", "i", "drop", "crop"), 
+				env = environment()) # postpone every arguments > 3 to after reading cells
+	}
 }
 
 # shrink bbox with e * width in each direction
@@ -508,13 +577,19 @@ st_crop.stars_proxy = function(x, y, ..., crop = TRUE, epsilon = sqrt(.Machine$d
 		# crop x:
 		dm[[ xd ]]$from = max(1, cr[1, 1], na.rm = TRUE)
 		dm[[ xd ]]$to = min(d_max[xd], cr[2, 1], na.rm = TRUE)
+		if(!is.null(dm[[ xd ]]$values))
+			dm[[ xd ]]$values = dm[[ xd ]]$values[dm[[ xd ]]$from:dm[[ xd ]]$to]
+		
 		# crop y:
-		if (dm[[ yd ]]$delta < 0)
+		if (!is.na(dm[[ yd ]]$delta) && dm[[ yd ]]$delta < 0) # FIXME: just subtract values to avoid NA miss?
 			cr[1:2, 2] = cr[2:1, 2]
 		dm[[ yd ]]$from = max(1, cr[1, 2], na.rm = TRUE)
 		dm[[ yd ]]$to = min(d_max[yd], cr[2, 2], na.rm = TRUE)
+		if(!is.null(dm[[ yd ]]$values))
+			dm[[ yd ]]$values = dm[[ yd ]]$values[dm[[ yd ]]$from:dm[[ yd ]]$to]
 	}
-	x = st_stars_proxy(x, dm, NA_value = attr(x, "NA_value"), resolutions = attr(x, "resolutions")) # crop to bb
+	x = st_stars_proxy(x, dm, NA_value = attr(x, "NA_value"), resolutions = attr(x, "resolutions"),
+		file_dim = attr(x, "file_dim")) # crop to bb
 	if (collect)
 		collect(x, match.call(), "st_crop", c("x", "y", "crop", "epsilon"),
 			env = environment(), ...) # crops further when realised
@@ -545,6 +620,19 @@ predict.stars_proxy = function(object, model, ...) {
 	structure(y, class = class(x))
 }
 
+#' @export
+st_normalize.stars_proxy = function(x, domain = c(0, 0, 1, 1), ...) {
+	stopifnot(all(domain == c(0,0,1,1)))
+	d = st_dimensions(x)
+	stopifnot(d[[1]]$from == 1, d[[2]]$from == 1)
+	x
+}
+
+#' @export
+image.stars_proxy <- function(x, ..., downsample = get_downsample(dim(x))) {
+    image(st_as_stars(x, downsample = downsample), ...)
+}
+
 #nocov start
 get_data_url = function(url, expr = NULL) {
 	if (!requireNamespace("httr", quietly = TRUE)) # GET, POST, PUT
